"use strict";

const {
  SCHEDULE_STATUS,
  DAP_SYNC_STATUS,
  TRANSFER_STATUS,
  CERTIFICATE_MONITOR_STATUS,
} = require("./app/constants");

class AppBootHook {
  constructor(app) {
    this.app = app;
    this.ctx = app.createAnonymousContext();
  }

  async willReady() {
    // =============== 监控定时任务开始 =============
    await this.app.logger.info("【初始化定时任务】开始...");
    // 查询启动状态的定时任务
    const schedules = await this.app.mysql.select("task_list", {
      where: { onlineType: SCHEDULE_STATUS.RUN },
    });
    // 循环注册定时任务
    console.log(schedules.length);
    schedules.length > 0 &&
      schedules.forEach(async (schedule) => {
        await this.app.logger.info(
          "【注册job】name：%s,",
          schedule.monitorName
        );
        await this.ctx.helper.generateSchedule(schedule);
      });
    await this.app.logger.info(
      "【初始化定时任务】初始化定时任务: %d，结束...",
      schedules.length
    );
    // =============== 监控定时任务结束 =============

    // ======== dap数据同步定时任务开始 ===========
    await this.app.logger.info("【初始化DAP数据同步定时任务】开始...");
    // 查询启动状态的定时任务
    const DapSyncSchedules = await this.app.mysql.select("burying_basic", {
      where: { DapSync: DAP_SYNC_STATUS.SYNC },
    });
    // 循环注册定时任务
    DapSyncSchedules.length > 0 &&
      DapSyncSchedules.forEach(async (schedule) => {
        await this.app.logger.info(
          "【注册的埋点cateogry】name：%s,",
          schedule.DapCategory
        );
        await this.ctx.helper.generateDapSyncSchedule(schedule);
      });
    await this.app.logger.info(
      "【初始化定时任务】初始化定时任务: %d，结束...",
      DapSyncSchedules.length
    );
    // ======== dap数据同步定时任务结束 ===========

    // ======== 转化率监控报警定时任务开始 ===========
    await this.app.logger.info("【初始化转化率监控报警定时任务】开始...");
    // 查询启动状态的定时任务
    const transferSchedules = await this.app.mysql.select("transfer", {
      where: { isAlert: TRANSFER_STATUS.on },
    });
    // 循环注册定时任务
    transferSchedules.length > 0 &&
      transferSchedules.forEach(async (schedule) => {
        await this.app.logger.info(
          "【注册的转化id】id：%s,",
          schedule.transferId
        );
        await this.ctx.helper.generateTransferSchedule(schedule);
      });
    await this.app.logger.info(
      "【初始化转化率监控报警定时任务】: %d，结束...",
      transferSchedules.length
    );
    // ======== 转化率监控报警定时任务结束 ===========


    // ========  证书监控报警定时任务开始 ========


    await this.app.logger.info("【初始化证书监控报警定时任务】开始...");
    // 查询启动状态的定时任务
    const certificateMonitorSchedules = await this.app.mysql.select("certificate_monitor", {
      where: { isOnline: CERTIFICATE_MONITOR_STATUS.on },
    });
    // 循环注册定时任务
    certificateMonitorSchedules.length > 0 &&
    certificateMonitorSchedules.forEach(async (schedule) => {
        await this.app.logger.info(
          "【注册的证书监控id】id：%s,",
          schedule.certificateMonitorId
        );
        await this.ctx.helper.generateCertificateMonitorSchedule(schedule);
      });
    await this.app.logger.info(
      "【初始化证书监控报警定时任务】: %d，结束...",
      certificateMonitorSchedules.length
    );
    // =======  证书监控报警定时任务结束 ======  
  }

  async beforeClose() {
    // ===============监控定时任务开始 =============

    await this.app.logger.info("【销毁定时任务】开始...");
    const scheduleStacks = await this.app.scheduleStacks;
    Reflect.ownKeys(scheduleStacks).forEach(async (key) => {
      await this.ctx.helper.cancelSchedule(key);
    });
    await this.app.logger.info(
      "【销毁定时任务】销毁定时任务数: %d，结束...",
      Reflect.ownKeys(scheduleStacks).length
    );
    // ===============监控定时任务结束 =============

    // ===============DAP数据同步任务开始 ===============
    await this.app.logger.info("【销毁定时任务】开始...");
    const dapscheduleStacks = await this.app.dapscheduleStacks;
    Reflect.ownKeys(dapscheduleStacks).forEach(async (key) => {
      await this.ctx.helper.cancelDapSchedule(key);
    });
    await this.app.logger.info(
      "【销毁定时任务】销毁定时任务数: %d，结束...",
      Reflect.ownKeys(dapscheduleStacks).length
    );

    // ===============DAP数据同步任务结束 ===============

    // ===============转化率监控报警任务开始 ===============
    await this.app.logger.info("【销毁定时任务】开始...");
    const transferStacks = await this.app.transferStacks;
    Reflect.ownKeys(transferStacks).forEach(async (key) => {
      const splitArr = key.split("&");
      await this.ctx.helper.cancelTransferSchedule(splitArr[0], splitArr[1]);
    });
    await this.app.logger.info(
      "【销毁定时任务】销毁定时任务数: %d，结束...",
      Reflect.ownKeys(dapscheduleStacks).length
    );

    // ===============转化率监控报警任务结束 ===============

    // =============== 证书监控报警任务开始 ==============
    await this.app.logger.info("【销毁定时任务】开始...");
    const certificateStacks = await this.app.certificateStacks;
    Reflect.ownKeys(certificateStacks).forEach(async (key) => {
      const splitArr = key.split("&");
      await this.ctx.helper.cancelCertificateMonitorSchedule(splitArr[0], splitArr[1]);
    });
    await this.app.logger.info(
      "【销毁定时任务】销毁定时任务数: %d，结束...",
      Reflect.ownKeys(certificateStacks).length
    );

    // =============== 证书监控报警任务结束 ==============

    
  }
}

module.exports = AppBootHook;
