"use strict";

const Controller = require("egg").Controller;
const { DAP_SYNC_STATUS } = require("../constants");
class BuryingPointController extends Controller {
  // 获取埋点列表
  async getBuryingList() {
    const { ctx } = this;
    let { current = "1", pageSize = "10" } = ctx.request.query;
    try {
      const sql = `
      select count(*) as num from (select distinct burying_basic.buryingId
       FROM 
       burying_basic left join bury_list on burying_basic.buryingId=bury_list.buryingId) as total
     `;
      const sql2 = `select burying_basic.*,sum(bury_list.type=2) as DapPointNum, sum(bury_list.type=1) as GaPointNum, projects.projectName
        from burying_basic
        left join bury_list on burying_basic.buryingId=bury_list.buryingId
        left join projects on burying_basic.projectid=projects.projectId
       group by burying_basic.buryingId
       ORDER BY burying_basic.buryingId DESC LIMIT ${
         (current - 1) * pageSize
       },${pageSize}`;
      const num = await ctx.app.mysql.query(sql);
      const list = await ctx.app.mysql.query(sql2);
      ctx.body = {
        msg: "",
        status: 200,
        data: {
          list,
          total: num[0].num,
          pages: Math.ceil(num[0].num / pageSize),
        },
      };
    } catch (error) {
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 获取所有DapAppName列表
  async getAppNameList() {
    const { ctx } = this;
    try {
      const res = await ctx.app.mysql.select("burying_basic", {
        columns: ["DapAppName"],
      });
      // 数组去重
      var result = [];
      var obj = {};
      for (var i = 0; i < res.length; i++) {
        if (!obj[res[i].DapAppName]) {
          result.push(res[i]);
          obj[res[i].DapAppName] = true;
        }
      }
      ctx.body = {
        msg: "",
        status: 200,
        data: result,
      };
    } catch (error) {
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 删除埋点
  async buryingDelete() {
    const { ctx } = this;
    const { buryingId, DapCategory, DapSync } = ctx.request.body;
    const conn = await this.app.mysql.beginTransaction();
    try {
      if (Number(DapSync) === 1 && DapCategory) {
        await this.ctx.helper.cancelDapSchedule(DapCategory);
      }
      await this.app.mysql.delete("bury_list", {
        buryingId: Number(buryingId),
      });
      await this.app.mysql.delete("burying_basic", {
        buryingId: Number(buryingId),
      });
      await conn.commit();
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      await conn.rollback(); // rollback call won't throw err
      if (Number(DapSync) === 1 && DapCategory) {
        await this.ctx.helper.generateDapSyncSchedule(DapCategory);
      }
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // ga埋点配置修改
  async GAConfigUpdate() {
    const { ctx } = this;
    const { buryingId, GaAppName, GACategory, GaScriptConfig, GaBuryingList } =
      ctx.request.body;
    const conn = await this.app.mysql.beginTransaction();
    try {
      // 先清掉埋点列表的数据
      await this.app.mysql.delete("bury_list", {
        type: 1,
        buryingId: Number(buryingId),
      });
      // 在全量设置进去
      if (GaBuryingList && GaBuryingList.length > 0) {
        await this.app.mysql.insert("bury_list", GaBuryingList);
      }
      // 更新basic主表数据
      await this.app.mysql.update(
        "burying_basic",
        {
          buryingId: Number(buryingId),
          GaAppName,
          GACategory,
          GaScriptConfig,
        },
        { where: { buryingId: Number(buryingId) } }
      );
      await conn.commit();
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      await conn.rollback(); // rollback call won't throw err
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // Dap埋点配置修改
  async DapConfigUpdate() {
    const { ctx } = this;
    const {
      buryingId,
      DapAppName,
      DapAppVersion,
      DapPackageName,
      DapCategory,
      DapBuryingList,
    } = ctx.request.body;
    const conn = await this.app.mysql.beginTransaction();
    try {
      //   // 先清掉埋点列表的数据
      await this.app.mysql.delete("bury_list", {
        type: 2,
        buryingId: Number(buryingId),
      });
      console.log(1);
      // 在全量设置进去
      if (DapBuryingList && DapBuryingList.length > 0) {
        await this.app.mysql.insert("bury_list", DapBuryingList);
      }
      console.log(2);
      // 更新basic主表数据
      await this.app.mysql.update(
        "burying_basic",
        {
          buryingId: Number(buryingId),
          DapAppName,
          DapAppVersion,
          DapPackageName,
          DapCategory,
        },
        { where: { buryingId: Number(buryingId) } }
      );
      console.log(3);
      await conn.commit();
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      await conn.rollback(); // rollback call won't throw err
      if (err.sqlMessage.indexOf("Duplicate") > -1) {
        return (ctx.body = {
          msg: "操作失败" + err.sqlMessage,
          status: 100012,
          data: {},
        });
      }
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 埋点基本信息新增/修改
  async baseInfoUpdate() {
    const { ctx } = this;
    const { buryingId, buryingCategory, projectId } = ctx.request.body;
    if (!buryingId) {
      // 新增
      try {
        await this.app.mysql.insert("burying_basic", {
          buryingCategory,
          projectId,
          DapSync: 0,
        });
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {},
        });
      } catch (error) {
        if (error.sqlMessage.indexOf("Duplicate") > -1) {
          return (ctx.body = {
            msg: "操作失败" + error.sqlMessage,
            status: 100012,
            data: {},
          });
        }
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    } else {
      // 修改
      try {
        await this.app.mysql.update(
          "burying_basic",
          {
            buryingId,
            buryingCategory,
            projectId,
            DapSync: 0,
          },
          { where: { buryingId: buryingId } }
        );
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {},
        });
      } catch (error) {
        if (error.sqlMessage.indexOf("Duplicate") > -1) {
          return (ctx.body = {
            msg: "操作失败" + error.sqlMessage,
            status: 100012,
            data: {},
          });
        }
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    }
  }
  // 获取埋点详情
  async buryingDetail() {
    const { ctx } = this;
    const { buryingId } = ctx.request.query;
    if (!buryingId) {
      return (ctx.body = {
        msg: "操作失败",
        status: 100,
        data: {},
      });
    } else {
      try {
        const detail = await this.app.mysql.get("burying_basic", {
          buryingId: Number(buryingId),
        });
        const burylist = await this.app.mysql.select("bury_list", {
          where: { buryingId: buryingId },
        });
        detail.burylist = burylist;
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {
            ...detail,
          },
        });
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    }
  }
  // 获取埋点统计的详细数据  三表联查数据
  async buryingStatis() {
    function handleWhere(queryParams) {
      where = " where ";
      const map = ["projectName"]; //项目表的查询条件
      const map2 = ["buryingCategory", "DapAppName"]; // 埋点基础表的查询条件

      Object.keys(queryParams).forEach((key, index) => {
        if (
          index != Object.keys(queryParams).length - 1 &&
          Object.keys(queryParams).length != 1
        ) {
          if (map.includes(key)) {
            where =
              where +
              `projects.${key} LIKE '%${decodeURIComponent(
                queryParams[key]
              )}%' AND `;
          } else if (map2.includes(key)) {
            where =
              where +
              `burying_basic.${key} LIKE '%${decodeURIComponent(
                queryParams[key]
              )}%' AND `;
          } else {
            where =
              where +
              `bury_list.${key} LIKE '%${decodeURIComponent(
                queryParams[key]
              )}%' AND `;
          }
        } else {
          if (map.includes(key)) {
            where =
              where +
              `projects.${key} LIKE '%${decodeURIComponent(
                queryParams[key]
              )}%'`;
          } else if (map2.includes(key)) {
            where =
              where +
              `burying_basic.${key} LIKE '%${decodeURIComponent(
                queryParams[key]
              )}%'`;
          } else {
            where =
              where +
              `bury_list.${key} LIKE '%${decodeURIComponent(
                queryParams[key]
              )}%'`;
          }
        }
      });
    }
    const { ctx } = this;
    let { current = "1", pageSize = "10", ...queryParams } = ctx.request.query;
    current = Number(current);
    pageSize = Number(pageSize);
    let where = "";
    if (JSON.stringify(queryParams) != "{}") {
      handleWhere(queryParams);
    } else {
      where = "";
    }
    try {
      const sql = `
      SELECT 
      count(*) as num FROM bury_list 
      left join burying_basic on bury_list.buryingId=burying_basic.buryingId  
      left join projects on burying_basic.projectId=projects.projectId ${where}
      `;
      const sql2 = `
      SELECT 
         * FROM bury_list 
         left join burying_basic on bury_list.buryingId=burying_basic.buryingId  
         left join projects on burying_basic.projectId=projects.projectId ${where} ORDER BY burying_basic.buryingId DESC LIMIT ${
        (current - 1) * pageSize
      },${pageSize}`;
      const num = await ctx.app.mysql.query(sql);
      const list = await ctx.app.mysql.query(sql2);
      ctx.body = {
        msg: "",
        status: 200,
        data: {
          list,
          total: num[0].num,
          pages: Math.ceil(num[0].num / pageSize),
        },
      };
    } catch (error) {
      console.log(error);
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 根据dapcategory 查询对应的埋点category id => buryiingId
  async getBuryIdBydDapcategory() {
    const { ctx } = this;
    const { DapCategory } = ctx.request.query;
    try {
      const detail = await this.app.mysql.get("burying_basic", {
        DapCategory: DapCategory,
      });
      console.log(detail);
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {
          buryingId: detail?.buryingId || "",
        },
      });
    } catch (error) {
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 获取Dap沙盘统计列表
  async DapStatistics() {
    const { ctx } = this;
    const { current: pageIdx, pageSize, ...queryParams } = ctx.request.query;
    try {
      const implaPath = this.config.implaPath;
      const data = {
        pageIdx: Number(pageIdx),
        pageSize: Number(pageSize),
        ...queryParams,
      };
      const result = await ctx.curl(`${implaPath}/getDotStatistic`, {
        method: "POST",
        contentType: "json",
        timeout: 10000,
        data,
        dataType: "json",
      });
      if (result.status === 200 && result.data.code === 200) {
        const list = result.data.data;
        const totalCount = result.data.totalCount;
        ctx.body = {
          msg: "",
          status: 200,
          data: {
            list,
            total: totalCount,
          },
        };
      } else {
        return (ctx.body = {
          msg: "操作失败" + e,
          status: 100,
          data: {},
        });
      }
    } catch (e) {
      return (ctx.body = {
        msg: "操作失败" + e,
        status: 100,
        data: {},
      });
    }
  }
  // 重新入队执行队列
  async reloadSync() {
    const { ctx } = this;
    const { batchId, appName, category, events, opdate } = ctx.request.body;
    try {
      const implaPath = this.config.implaPath;
      const data = {
        batchId,
        appName,
        category,
        events,
        opdate,
      };
      const result = await ctx.curl(`${implaPath}/pushQueryTask`, {
        method: "POST",
        contentType: "json",
        timeout: 10000,
        data,
        dataType: "json",
      });
      if (result.status === 200 && result.data.code === 200) {
        const list = result.data.data;
        const totalCount = result.data.totalCount;
        ctx.body = {
          msg: "",
          status: 200,
          data: {
            list,
            total: totalCount,
          },
        };
      } else {
        return (ctx.body = {
          msg: "操作失败",
          status: 100,
          data: {},
        });
      }
    } catch (e) {
      return (ctx.body = {
        msg: "操作失败" + e,
        status: 100,
        data: {},
      });
    }
  }
  // 是否同步每日DAP数据
  async DapSync() {
    const { ctx } = this;
    const { buryingId, DapSync } = ctx.request.body;
    try {
      if (DapSync === DAP_SYNC_STATUS.SYNC) {
        try {
          const result = await this.app.mysql.update(
            "burying_basic",
            {
              DapSync: DapSync,
            },
            { where: { buryingId: buryingId } }
          );
          if (result.affectedRows === 1) {
            const scheduleItem = await this.app.mysql.get("burying_basic", {
              buryingId: buryingId,
            });
            await this.ctx.helper.generateDapSyncSchedule(scheduleItem);
            return (ctx.body = {
              msg: "操作成功",
              status: 200,
              data: {},
            });
          }
        } catch (error) {
          return (ctx.body = {
            msg: "操作失败" + error.sqlMessage,
            status: 100,
            data: {},
          });
        }
      } else if (DapSync === DAP_SYNC_STATUS.UNSYNC) {
        try {
          const result = await this.app.mysql.update(
            "burying_basic",
            {
              DapSync: DapSync,
            },
            { where: { buryingId: buryingId } }
          );
          if (result.affectedRows === 1) {
            const scheduleItem = await this.app.mysql.get("burying_basic", {
              buryingId: buryingId,
            });
            await this.ctx.helper.cancelDapSchedule(scheduleItem.DapCategory);
            return (ctx.body = {
              msg: "操作成功",
              status: 200,
              data: {},
            });
          }
        } catch (error) {
          return (ctx.body = {
            msg: "操作失败" + error.sqlMessage,
            status: 100,
            data: {},
          });
        }
      }
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (error) {
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 查询impla任务队列
  async impalaTasklist() {
    const { ctx } = this;
    const { current: pageIdx, pageSize, ...queryParams } = ctx.request.query;
    try {
      const implaPath = this.config.implaPath;
      const data = {
        pageIdx: Number(pageIdx),
        pageSize: Number(pageSize),
        ...queryParams,
      };
      const result = await ctx.curl(`${implaPath}/getQueryTaskList`, {
        method: "POST",
        contentType: "json",
        timeout: 10000,
        data,
        dataType: "json",
      });
      if (result.status === 200 && result.data.code === 200) {
        const list = result.data.data;
        const totalCount = result.data.totalCount;
        ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {
            list,
            total: totalCount,
          },
        };
      } else {
        return (ctx.body = {
          msg: "操作失败" + e,
          status: 100,
          data: {},
        });
      }
    } catch (e) {
      return (ctx.body = {
        msg: "操作失败" + e,
        status: 100,
        data: {},
      });
    }
  }
  // 是否可删除、保存埋点
  async canOperateDap() {
    const { ctx } = this;
    const { DapAppName } = ctx.request.body;
    let canDeleteDap;
    try {
      const items = await this.app.mysql.select("product_model_basic", {
        where: { appName: DapAppName },
      });
      if (items && items.length > 0) {
        canDeleteDap = true;
      } else {
        canDeleteDap = false;
      }
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {
          canDeleteDap: canDeleteDap,
        },
      });
    } catch (e) {
      return (ctx.body = {
        msg: "操作失败" + e,
        status: 100,
        data: {},
      });
    }
  }
}

module.exports = BuryingPointController;
