"use strict";
const Controller = require("egg").Controller;
const { CERTIFICATE_MONITOR_STATUS } = require("../constants");

class certificateMonitorController extends Controller {
  //  证书监控列表
  async certificateMonitorList() {
    const { ctx } = this;
    let { current = 1, pageSize = 10, ...queryParmas } = ctx.request.query;
    current = Number(current);
    pageSize = Number(pageSize);
    try {
      const list = await ctx.app.mysql.select("certificate_monitor", {
        where: { ...queryParmas },
        columns: [
          "certificateMonitorName",
          "certificateMonitorDomain",
          "certificateDays",
          "certificateMonitorId",
          "createTime",
          "updateTime",
          "isOnline",
        ],
        limit: pageSize, // 返回数据量
        offset: (current - 1) * pageSize, // 数据偏移量
      });
      const sql = `
      SELECT 
        count(*) as num 
      FROM 
      certificate_monitor
    `;
    const num = await ctx.app.mysql.query(sql);

      
      ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {
          list,
          total: num[0].num,
        },
      };
    } catch (err) {
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 新增 / 编辑证书监控
  async certificateMonitorUpdate() {
    const { ctx } = this;
    const {
      certificateMonitorId,
      certificateMonitorName,
      certificateMonitorDomain,
    } = ctx.request.body;
    if (!certificateMonitorId) {
      // 新增
      try {
        await this.app.mysql.insert("certificate_monitor", {
          certificateMonitorName,
          certificateMonitorDomain,
          isOnline: CERTIFICATE_MONITOR_STATUS.off, // 默认新增是下线状态需要手动开启
          createTime: new Date().getTime(),
        });
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {},
        });
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    } else {
      // 修改
      try {
        await this.app.mysql.update(
          "certificate_monitor",
          {
            certificateMonitorName,
            certificateMonitorDomain,
            updateTime: new Date().getTime(),
          },
          { where: { certificateMonitorId: certificateMonitorId } }
        );
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {},
        });
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    }
  }
  // 上线 / 下线证书监控
  async certificateMonitorOnline() {
    const { ctx } = this;
    const { certificateMonitorId, isOnline } = ctx.request.body;
    if (isOnline === 0) {
      // 下线
      try {
        const result = await this.app.mysql.update(
          "certificate_monitor",
          {
            isOnline: CERTIFICATE_MONITOR_STATUS.off,
          },
          { where: { certificateMonitorId: certificateMonitorId } }
        );
        if (result.affectedRows === 1) {
          const scheduleItem = await this.app.mysql.get("certificate_monitor", {
            certificateMonitorId: certificateMonitorId,
          });
          await this.ctx.helper.cancelCertificateMonitorSchedule(scheduleItem.certificateMonitorName, scheduleItem.certificateMonitorId);
          return (ctx.body = {
            msg: "操作成功",
            status: 200,
            data: {},
          });
        }
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    } else if (isOnline === 1) {
      // 上线
      try {
        const result = await this.app.mysql.update(
          "certificate_monitor",
          {
            isOnline: CERTIFICATE_MONITOR_STATUS.on,
          },
          { where: { certificateMonitorId: certificateMonitorId } }
        );
        if (result.affectedRows === 1) {
          const scheduleItem = await this.app.mysql.get("certificate_monitor", {
            certificateMonitorId: certificateMonitorId,
          });
          await this.ctx.helper.generateCertificateMonitorSchedule(scheduleItem);
          return (ctx.body = {
            msg: "操作成功",
            status: 200,
            data: {},
          });
        }
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    }
  }
}

module.exports = certificateMonitorController;
