"use strict";

const { ONLINETYPE } = require("../constants");

const Controller = require("egg").Controller;

class MonitorController extends Controller {
  // 测试监控任务配置是否正确
  async testMonitor() {
    const { ctx } = this;
    const { monitorUrl, resultRules, contentType } = ctx.request.body;
    const taskItem = ctx.request.body;
    const res = await this.service.curlService.tarsServerHttp(monitorUrl, {
      dataType: contentType,
    });
    if (res.status !== 200 || (res.data.code && res.data.code !== 0)) {
      return (ctx.body = {
        msg: "操作失败",
        status: 11001,
        data: res,
      });
    } else {
      if (contentType === "json") {
        if (resultRules) {
          if (res.data && JSON.stringify(res.data).match(resultRules)) {
            return (ctx.body = {
              msg: "操作成功",
              status: 200,
              data: res,
            });
          } else {
            return (ctx.body = {
              msg: "操作失败",
              status: 11001,
              data: res,
            });
          }
        } else {
          return (ctx.body = {
            msg: "操作成功",
            status: 200,
            data: res,
          });
        }
      } else if (contentType === "text") {
        if (resultRules) {
          if (res.data && res.data.match(resultRules)) {
            return (ctx.body = {
              msg: "操作成功",
              status: 200,
              data: res,
            });
          } else {
            return (ctx.body = {
              msg: "操作失败",
              status: 11001,
              data: res,
            });
          }
        } else {
          return (ctx.body = {
            msg: "操作成功",
            status: 200,
            data: res,
          });
        }
      }
    }
  }
  // 获取监控任务列表
  async getMonitorList() {
    function handleWhere(queryParams) {
      where = " where ";
      const map = ["projectName"];
      Object.keys(queryParams).forEach((key, index) => {
        if (
          index != Object.keys(queryParams).length - 1 &&
          Object.keys(queryParams).length != 1
        ) {
          if (map.includes(key)) {
            where = where + `projects.${key} LIKE '%${queryParams[key]}%' AND `;
          } else {
            where =
              where + `task_list.${key} LIKE '%${queryParams[key]}%' AND `;
          }
        } else {
          if (map.includes(key)) {
            where = where + `projects.${key} LIKE '%${queryParams[key]}%'`;
          } else {
            where = where + `task_list.${key} LIKE '%${queryParams[key]}%'`;
          }
        }
      });
    }
    const { ctx } = this;
    let { current = "1", pageSize = "10", ...queryParams } = ctx.request.query;
    current = Number(current);
    pageSize = Number(pageSize);
    let where = "";
    if (JSON.stringify(queryParams) != "{}") {
      handleWhere(queryParams);
    } else {
      where = "";
    }
    try {
      const sql =
        JSON.stringify(queryParams) != "{}"
          ? `
       SELECT 
         count(*) as num 
       FROM 
       task_list inner join projects on task_list.projectId=projects.projectId ${where}
     `
          : `
     SELECT 
       count(*) as num 
     FROM 
     task_list inner join projects on task_list.projectId=projects.projectId
   `;

      const sql2 = `
      select * from task_list inner join projects on task_list.projectId=projects.projectId ${where} ORDER BY monitorId DESC LIMIT ${
        (current - 1) * pageSize
      },${pageSize}`;
      const num = await ctx.app.mysql.query(sql);
      const list = await ctx.app.mysql.query(sql2);
      ctx.body = {
        msg: "",
        status: 200,
        data: {
          list,
          total: num[0].num,
          pages: Math.ceil(num[0].num / pageSize),
        },
      };
    } catch (error) {
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 更新监控任务
  async updateMonitor() {
    const { ctx } = this;
    const {
      monitorId,
      monitorName,
      monitorUrl,
      resultRules,
      contentType,
      timerNum,
      email,
      projectId,
      monitorLevel
    } = ctx.request.body;
    if (!monitorId) {
      // 新增
      try {
        await this.app.mysql.insert("task_list", {
          monitorName,
          monitorUrl,
          monitorLevel,
          onlineType: ONLINETYPE.OFFLINE, // 默认新增是下线状态  需要手动开启
          resultRules,
          contentType,
          createTime: new Date().getTime(),
          timerNum,
          email,
          projectId,
        });
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {},
        });
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    } else {
      // 修改
      try {
        await this.app.mysql.update(
          "task_list",
          {
            monitorId,
            monitorName,
            onlineType: ONLINETYPE.OFFLINE, // 只有下线的情况下才能编辑
            monitorUrl,
            monitorLevel,
            contentType,
            resultRules,
            createTime: new Date().getTime(),
            timerNum,
            email,
            projectId,
            updateTime: new Date().getTime(),
          },
          { where: { monitorId: monitorId } }
        );
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {},
        });
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    }
  }
  // 监控上线下线操作
  async onlineMonitor() {
    const { ctx } = this;
    const { monitorId, onlineType } = ctx.request.body;

    const taskItem = await this.app.mysql.get("task_list", {
      monitorId: monitorId,
    });
    if (onlineType === 3) {
      // 下线
      try {
        const result = await this.app.mysql.update(
          "task_list",
          {
            onlineType: ONLINETYPE.OFFLINE,
          },
          { where: { monitorId: monitorId } }
        );
        if (result.affectedRows === 1) {
          const scheduleItem = await this.app.mysql.get("task_list", {
            monitorId: monitorId,
          });
          await this.ctx.helper.cancelSchedule(scheduleItem.monitorName);
          return (ctx.body = {
            msg: "操作成功",
            status: 200,
            data: {},
          });
        }
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    } else if (onlineType === 2) {
      // 上线
      try {
        const result = await this.app.mysql.update(
          "task_list",
          {
            onlineType: ONLINETYPE.ONLINE,
          },
          { where: { monitorId: monitorId } }
        );
        if (result.affectedRows === 1) {
          const scheduleItem = await this.app.mysql.get("task_list", {
            monitorId: monitorId,
          });
          await this.ctx.helper.generateSchedule(scheduleItem);
          return (ctx.body = {
            msg: "操作成功",
            status: 200,
            data: {},
          });
        }
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    }
  }
  // 监控历史记录
  async historyList() {
    const { ctx } = this;
    function handleWhere(queryParams) {
      where = " where ";
      const map = ["projectName"];
      Object.keys(queryParams).forEach((key, index) => {
        if (
          Object.keys(queryParams).length != 1 &&
          index != Object.keys(queryParams).length - 1
        ) {
          if (map.includes(key)) {
            where =
              where +
              `projects.${key} LIKE'%${decodeURIComponent(
                queryParams[key]
              )}%' AND `;
          } else {
            if (key === "sendResult" || key === "checkResult") {
              queryParams[key] = Number(queryParams[key]);
            } else {
              queryParams[key] = decodeURIComponent(queryParams[key]);
            }
            where =
              where + `task_history.${key} LIKE '%${queryParams[key]}%' AND `;
          }
        } else {
          if (map.includes(key)) {
            where =
              where +
              `projects.${key} LIKE '%${decodeURIComponent(
                queryParams[key]
              )}%'`;
          } else {
            if (key === "sendResult" || key === "checkResult") {
              queryParams[key] = Number(queryParams[key]);
            } else {
              queryParams[key] = decodeURIComponent(queryParams[key]);
            }
            where = where + `task_history.${key} LIKE '%${queryParams[key]}%'`;
          }
        }
      });
    }
    let { current, pageSize, ...queryParams } = ctx.request.query;
    current = Number(current);
    pageSize = Number(pageSize);
    let where = "";
    if (JSON.stringify(queryParams) != "{}") {
      handleWhere(queryParams);
    } else {
      where = "";
    }
    try {
      const sql =
        JSON.stringify(queryParams) != "{}"
          ? `
      SELECT 
        count(*) as num 
      FROM 
      task_history inner join projects on task_history.projectId=projects.projectId ${where}
    `
          : `
    SELECT 
      count(*) as num 
    FROM 
    task_history inner join projects on task_history.projectId=projects.projectId
  `;
      const sql2 = `
      select * from task_history inner join projects on task_history.projectId=projects.projectId ${where} ORDER BY monitorhistoryId DESC LIMIT ${
        (current - 1) * pageSize
      },${pageSize}`;
      const num = await ctx.app.mysql.query(sql);
      const list = await ctx.app.mysql.query(sql2);
      ctx.body = {
        msg: "",
        status: 200,
        data: {
          list,
          total: num[0].num,
          pages: Math.ceil(num[0].num / pageSize),
        },
      };
    } catch (error) {
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 监控失败率
  async failureMonitor() {
    const { ctx } = this;
    try {
      let resData = [];
      const sql = `select count(1) as sum,sum(checkResult) as success,date_format(task_history.createTime, '%Y-%m-%d') as Date  from task_history where createTime >= date(now()) - interval 7 day group by date_format(task_history.createTime, '%Y-%m-%d');`;
      const list = await ctx.app.mysql.query(sql);
      if (list && list.length > 0) {
        resData = list.map((item) => {
          return {
            failData: Number(item.sum)
              ? Number(
                  (((item.sum - item.success) / item.sum) * 100).toFixed(2)
                )
              : 0,
            Date: item.Date,
          };
        });
      }
      ctx.body = {
        msg: "",
        status: 200,
        data: {
          resData,
        },
      };
    } catch (e) {
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {
          resData: [],
        },
      });
    }
  }
}

module.exports = MonitorController;
