const Controller = require("egg").Controller;
const dayjs = require("dayjs");
class ProductModelController extends Controller {
  // 产品模型新增
  async productModelAdd() {
    const { ctx } = this;
    const {
      modelId, // 前端生成的uuid
      transferList, // 转化路径列表
      treeList, // 树列表
      modelName, // 模型名称
      appName, // appName
      materialList, // 素材列表
      choosedMaterialList, // 已选择的素材列表
    } = ctx.request.body;
    treeList.forEach((item) => {
      item.modelId = modelId;
    });
    transferList.forEach((item) => {
      item.modelId = modelId;
      item.isAlert = 0;
    });
    const conn = await this.app.mysql.beginTransaction();
    try {
      // insert 产品模型基础表
      await this.app.mysql.insert("product_model_basic", {
        modelId,
        modelName,
        appName,
        materialList: JSON.stringify(materialList),
        choosedMaterialList: JSON.stringify(choosedMaterialList),
      });
      // insert 产品模型树结构表
      await this.app.mysql.insert("product_model_tree", treeList);
      // insert 转化表
      await this.app.mysql.insert("transfer", transferList);
      await conn.commit();
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      await conn.rollback(); // rollback call won't throw err

      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 产品模型编辑
  async productModelEdit() {
    const { ctx } = this;
    const {
      modelId,
      transferList, // 转化路径列表
      treeList, // 树列表
      tranferIds, // 需要删除的id组合
      modelName, // 模型名称
      appName, // appName
      materialList, // 素材列表
      choosedMaterialList, // 已选择的素材列表
    } = ctx.request.body;
    treeList.forEach((item) => {
      item.modelId = modelId;
    });
    transferList.forEach((item) => {
      item.modelId = modelId;
      item.isAlert = 0;
    });
    const conn = await this.app.mysql.beginTransaction();
    try {
      // update 产品模型基础表
      await this.app.mysql.update(
        "product_model_basic",
        {
          modelId,
          modelName,
          appName,
          materialList: JSON.stringify(materialList),
          choosedMaterialList: JSON.stringify(choosedMaterialList),
        },
        { where: { modelId: modelId } }
      );
      // 先删掉 产品模型树结构
      await this.app.mysql.delete("product_model_tree", {
        modelId: modelId,
      });
      // insert 产品模型树结构表
      await this.app.mysql.insert("product_model_tree", treeList);
      if (tranferIds && tranferIds.length > 0) {
        // 删除 transfer表中多余的记录
        await this.app.mysql.delete("transfer", {
          transferId: tranferIds,
        });
      }
      // insert 转化表
      if (transferList && transferList.length > 0) {
        await this.app.mysql.insert("transfer", transferList);
      }
      await conn.commit();
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      await conn.rollback(); // rollback call won't throw err

      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 产品模型详情
  async productModelView() {
    const { ctx } = this;
    const { modelId, type } = ctx.request.query;
    try {
      // 基础数据
      const basicDetail = await this.app.mysql.get("product_model_basic", {
        modelId: modelId,
      });
      basicDetail.choosedMaterialList = JSON.parse(
        basicDetail.choosedMaterialList
      );
      basicDetail.materialList = JSON.parse(basicDetail.materialList);
      // 模型树数据
      const modelTreeDetail = await this.app.mysql.select(
        "product_model_tree",
        {
          where: {
            modelId: modelId,
          },
        }
      );
      if (type === "view") {
        // 查看时调impla微服务数据
        const events = [
          ...new Set(
            modelTreeDetail.map((item) => {
              return item.eventName;
            })
          ),
        ];
        const categorys = [
          ...new Set(
            modelTreeDetail.map((item) => {
              return item.category;
            })
          ),
        ];
        const implaPath = this.config.implaPath; // 微服务路径地址
        const data = {
          appNames: [basicDetail.appName],
          events: events,
          categorys: categorys,
          opdate: dayjs(new Date()).subtract(1, "day").format("YYYY-MM-DD"),
        };
        const res = await ctx.curl(`${implaPath}/getDotStatisticByEvts`, {
          method: "POST",
          contentType: "json",
          timeout: 10000,
          data,
          dataType: "json",
        });
        if (res.data.Result === 1) {
          modelTreeDetail.forEach((i) => {
            res.data.data.forEach((j) => {
              if (i.eventName === j.event) {
                i.val1 = j.val1;
                i.val2 = j.val2;
              }
            });
          });
        }
      }

      // 转化数据
      const transferDetail = await this.app.mysql.select("transfer", {
        where: {
          modelId: modelId,
        },
      });
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {
          ...basicDetail,
          modelTreeDetail,
          transferDetail,
        },
      });
    } catch (err) {
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 清空数据模型
  async productModelDelete() {
    const { ctx } = this;
    const { modelId } = ctx.request.body;
    try {
      const conn = await this.app.mysql.beginTransaction();
      // 查询当前数据模型下开启监控报警的定时任务
      const alertArr = await this.app.mysql.select("transfer", {
        where: {
          modelId: modelId,
          isAlert: 1,
        },
      });
      // 停掉当前开启的转化率监控报警任务
      alertArr.length > 0 &&
        alertArr.forEach((item) => {
          ctx.helper.cancelTransferSchedule(item.modelId, item.transferId);
        });
      // 清掉转化表中的数据
      await this.app.mysql.delete("transfer", {
        modelId: modelId,
      });
      // 清掉产品数据模型基础表
      await this.app.mysql.delete("product_model_basic", {
        modelId: modelId,
      });
      // 清掉产品数据模型树结构表
      await this.app.mysql.delete("product_model_tree", {
        modelId: modelId,
      });
      await conn.commit();
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      await conn.rollback(); // rollback call won't throw err
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 产品模型列表
  async productModelList() {
    const { ctx } = this;
    let { pageIndex = 1, pageSize = 10 } = ctx.request.query;
    pageIndex = Number(pageIndex);
    pageSize = Number(pageSize);
    try {
      const list = await ctx.app.mysql.select("product_model_basic", {
        columns: [
          "modelName",
          "appName",
          "materialList",
          "choosedMaterialList",
          "modelId",
        ],
        limit: pageSize, // 返回数据量
        offset: (pageIndex - 1) * pageSize, // 数据偏移量
      });
      const sql = `
      SELECT 
        count(*) as num 
      FROM 
      product_model_basic
    `;
      const num = await ctx.app.mysql.query(sql);
      ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {
          list,
          current: pageIndex,
          total: num[0].num,
          pages: Math.ceil(num[0].num / pageSize),
        },
      };
    } catch (err) {
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 获取产品模型树列表
  async getTree() {
    const { ctx } = this;
    const { modelId } = ctx.request.query;
    try {
      // 模型树数据
      const modelTreeDetail = await this.app.mysql.select(
        "product_model_tree",
        {
          where: {
            modelId: modelId,
          },
        }
      );
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {
          modelTreeDetail,
        },
      });
    } catch (err) {
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 获取节点数据的转化列表
  async getNodeDetail() {
    const { ctx } = this;
    const { transferList, appName } = ctx.request.body;
    try {
      const eventsArr = [];
      const categoryArr = [];
      transferList.forEach((item) => {
        eventsArr.push(item.fromEventName);
        eventsArr.push(item.toEventName);
        categoryArr.push(item.fromCategory);
        categoryArr.push(item.toCategory);
      });
      const events = [...new Set(eventsArr)];
      const categorys = [...new Set(categoryArr)];
      const implaPath = this.config.implaPath; // 微服务路径地址
      const data = {
        appNames: [appName],
        events: events,
        categorys: categorys,
        opdate: dayjs(new Date()).subtract(1, "day").format("YYYY-MM-DD"),
      };
      const res = await ctx.curl(`${implaPath}/getDotStatisticByEvts`, {
        method: "POST",
        contentType: "json",
        timeout: 10000,
        data,
        dataType: "json",
      });
      if (res.data.Result === 1) {
        transferList.forEach((i) => {
          res.data.data.forEach((j) => {
            if (i.fromEventName === j.event && i.fromCategory === j.category) {
              i.fromVal1 = j.val1;
              i.fromVal2 = j.val2;
            }
            if (i.toEventName === j.event && i.toCategory === j.category) {
              i.toVal1 = j.val1;
              i.toVal2 = j.val2;
            }
          });
          if (!i.fromVal1) {
            i.fromVal1 = 0;
          }
          if (!i.fromVal2) {
            i.fromVal2 = 0;
          }
          if (!i.toVal1) {
            i.toVal1 = 0;
          }
          if (!i.toVal2) {
            i.toVal2 = 0;
          }
        });
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {
            list: transferList,
          },
        });
      }
    } catch (e) {
      this.logger.info(`查询impla查询失败：${e}`);
      return (ctx.body = {
        msg: "操作失败" + e,
        status: 100,
        data: {},
      });
    }
  }
}
module.exports = ProductModelController;
