"use strict";

const Controller = require("egg").Controller;
class RoleController extends Controller {
  // 获取角色列表
  async roleList() {
    const { ctx } = this;
    try {
      const result = await this.app.mysql.select("roles", {
        columns: ["roleId", "roleName", "roleDesc"],
      });
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {
          list: result,
        },
      });
    } catch (err) {
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 新增角色
  async roleAdd() {
    const { ctx, app } = this;
    const { roleName, roleDesc } = ctx.request.body;
    try {
      let result;
      const row = {
        roleName,
        roleDesc,
        userMenus: JSON.stringify([]),
      };
      result = await app.mysql.insert("roles", row);
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      if (err.sqlMessage.indexOf("Duplicate") > -1) {
        return (ctx.body = {
          msg: "操作失败" + err.sqlMessage,
          status: 100024, // 角色名称不能重复
          data: {},
        });
      }
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 编辑角色
  async roleEdit() {
    const { ctx, app } = this;
    const { roleName, roleDesc, roleId } = ctx.request.body;
    try {
      let result;
      const rows = {
        roleName,
        roleDesc,
      };
      const options = {
        where: { roleId },
      };
      result = await app.mysql.update("roles", rows, options);
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      if (err.sqlMessage.indexOf("Duplicate") > -1) {
        return (ctx.body = {
          msg: "操作失败" + err.sqlMessage,
          status: 100024, // 角色名称不能重复
          data: {},
        });
      }
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 角色详情
  async roleDetail() {
    const { ctx } = this;
    const { roleId } = ctx.request.query;
    if (!roleId) {
      return (ctx.body = {
        msg: "操作失败",
        status: 100,
        data: {},
      });
    } else {
      try {
        const detail = await this.app.mysql.get("roles", {
          roleId: Number(roleId),
        });
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {
            ...detail,
          },
        });
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    }
  }
  // 删除角色
  async roleDelete() {
    const { ctx } = this;
    const { roleId } = ctx.request.body;
    const result = await this.app.mysql.select("users", {
      where: { roleId: Number(roleId) },
    });
    if (result.length > 0) {
      return (ctx.body = {
        msg: "操作失败",
        status: 100023, // 当前还有用户在使用的角色,不能删除
        data: {},
      });
    }
    await this.app.mysql.delete("roles", {
      roleId: Number(roleId),
    });
    try {
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 角色菜单权限配置
  async roleMenuConfig() {
    const { ctx, app } = this;
    const { roleId, userMenus } = ctx.request.body;
    let result;
    const rows = {
      userMenus: JSON.stringify(userMenus),
    };
    const options = {
      where: { roleId },
    };
    result = await app.mysql.update("roles", rows, options);

    try {
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (err) {
      return (ctx.body = {
        msg: "操作失败" + err.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
}

module.exports = RoleController;
