"use strict";

const Controller = require("egg").Controller;
const md5 = require("md5");

class UserController extends Controller {
  // 获取用户列表
  async getUserList() {
    const { ctx } = this;
    let { current = 1, pageSize = 10 } = ctx.request.query;
    current = Number(current);
    pageSize = Number(pageSize);
    try {
      const sql = `SELECT 
      count(*) as num 
    FROM 
    users left join roles on users.roleId=roles.roleId`;
      const sql2 = `
    select * from users left join roles on users.roleId=roles.roleId ORDER BY users.createAt DESC LIMIT ${
      (current - 1) * pageSize
    },${pageSize}`;
      const num = await ctx.app.mysql.query(sql);
      const list = await ctx.app.mysql.query(sql2);
      ctx.body = {
        msg: "",
        status: 200,
        data: {
          list,
          total: num[0].num,
          pages: Math.ceil(num[0].num / pageSize),
        },
      };
    } catch (error) {
      console.log(error);
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
  // 更新用户
  async addUser() {
    const { ctx, app } = this;
    let { ...body } = ctx.request.body;
    console.log(body);
    if (!body.userId) {
      body.password = md5("123456");
      body.createAt = Date.parse(new Date());
      body.lastEditAt = Date.parse(new Date());
      body.status = 1
      try {
        const result = await app.mysql.insert("users", body);
        // 判断插入成功
        const insertSuccess = result.affectedRows === 1;

        if (insertSuccess) {
          return (ctx.body = {
            msg: "操作成功",
            status: 200,
            data: "",
          });
        } else {
          return (ctx.body = {
            msg: "操作失败",
            status: 100,
            data: "",
          });
        }
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败：" + error.sqlMessage,
          status: 100,
          data: "",
        });
      }
    } else {
      try {
        await app.mysql.update(
          "users",
          {
            userName: body.userName,
            roleId: body.roleId,
            email: body.email,
            lastEditAt: Date.parse(new Date()),
          },
          { where: { userId: body.userId } }
        );
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: {},
        });
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败" + error.sqlMessage,
          status: 100,
          data: {},
        });
      }
    }
  }
  // 修改密码
  async editPassword() {
    const { ctx, app } = this;
    let { currentPasword, newPassword, userId } = ctx.request.body;

    const res = await ctx.app.mysql.select("users", {
      where: { userId },
      columns: ["password"],
    });

    // 如果当天密码正确
    if (res[0].password === md5(currentPasword)) {
      const row = {
        password: md5(newPassword),
      };
      const options = {
        where: {
          userId,
        },
      };

      try {
        const result = await app.mysql.update("users", row, options); // 更新 users 表中的密码
        // const insertSuccess = result.affectedRows === 1;
        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: "",
        });
      } catch (error) {
        return (ctx.body = {
          msg: "操作失败",
          status: 100,
          data: "",
        });
      }
    } else {
      return (ctx.body = {
        msg: "操作失败：密码错误",
        status: 100,
        data: "",
      });
    }
  }

  // 发送邮件
  async sendEmailCode() {
    const { ctx, app } = this;
    const code = Math.random().toFixed(6).slice(-6);
    const { userId, email } = ctx.request.body;

    const subject = "标题：测试邮件";
    const text = "文本：这是一封测试邮件";
    const html =
      "<h2>测试一下，乌拉~</h2><br/><p>验证码是：<b>" + code + "</b></p>";

    const has_send = await this.service.tool.sendMail(
      email,
      subject,
      text,
      html
    );
    console.log(has_send);
    if (has_send) {
      // 写入数据库
      const data = {
        userId,
        code,
        createAt: Date.parse(new Date()),
        consume: 0,
      };

      const result = await app.mysql.insert("codes", data);

      ctx.body = {
        message: "发送成功",
      };
      return;
    }
    ctx.body = {
      message: "发送失败",
    };
  }
  // 修改邮箱
  async editEmail() {
    const { ctx, app } = this;
    let { newEmial, code, userId } = ctx.request.body;
    code = Number(code);

    const emailRes = await ctx.app.mysql.select("codes", {
      where: { userId },
      columns: ["id", "code", "createAt", "consume"],
      orders: [["createAt", "desc"]], // 排序方式
    });
    // 判断验证码是否正确
    if (emailRes[0].code === code) {
      // 判断验证码是否使用
      if (emailRes[0].consume === 1) {
        return (ctx.body = {
          msg: "验证码已使用",
          status: 100,
          data: "",
        });
      }
      // 判断验证码是否过期
      const state =
        (Date.parse(new Date()) - emailRes[0].createAt) / 1000 / 60 > 10;
      if (state) {
        return (ctx.body = {
          msg: "验证码已过期",
          status: 100,
          data: "",
        });
      }

      const row1 = {
        email: newEmial,
      };
      const options1 = {
        where: {
          userId,
        },
      };
      const row2 = {
        consume: 1,
      };
      const options2 = {
        where: {
          id: emailRes[0].id,
        },
      };
      try {
        // 更新 users 表中的email
        await app.mysql.update("users", row1, options1);

        // 更新 codes 表中的使用状态
        await app.mysql.update("codes", row2, options2);

        return (ctx.body = {
          msg: "操作成功",
          status: 200,
          data: "",
        });
      } catch (error) {
        console.log(error);
        return (ctx.body = {
          msg: "操作失败：" + error.sqlMessage,
          status: 100,
          data: "",
        });
      }
    } else {
      return (ctx.body = {
        msg: "验证码错误",
        status: 100,
        data: "",
      });
    }
  }
  // 获取个人信息
  async getUserInfo() {
    const { ctx } = this;
    let { userId } = ctx.request.query;
    try {
      let detail;
      const userDetail = await ctx.app.mysql.select("users", {
        where: { userId },
        columns: ["userId", "userName", "roleId", "role", "email", "status"],
      });
      const roleDetail = await ctx.app.mysql.select("roles", {
        where: { roleId: userDetail[0].roleId },
      });
      detail = {
        ...userDetail[0],
        ...roleDetail[0],
      };
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {
          ...detail,
        },
      });
    } catch (error) {
      console.log(error)
      return (ctx.body = {
        msg: "操作失败：" + error.sqlMessage,
        status: 100,
        data: "",
      });
    }
  }
  async operateUser() {
    const { ctx } = this;
    let { userId, status } = ctx.request.body;
    try {
      await this.app.mysql.update(
        "users",
        {
          status: Number(status),
        },
        { where: { userId: userId } }
      );
      return (ctx.body = {
        msg: "操作成功",
        status: 200,
        data: {},
      });
    } catch (error) {
      return (ctx.body = {
        msg: "操作失败" + error.sqlMessage,
        status: 100,
        data: {},
      });
    }
  }
}

module.exports = UserController;
