"use strict";
const schedule = require("node-schedule");
const NodeUUID = require("node-uuid");
const {
  SCHEDULE_STATUS,
  DAP_SYNC_STATUS,
  TRANSFER_STATUS,
  CERTIFICATE_MONITOR_STATUS
} = require("../constants");
const {
  SCHEDULE_STACKS,
  DAP_SCHEDULE_STACKS,
  TRANSFE_STACKS,
  CERTIFICATE_STACKS,
} = require("../constants/redis");
module.exports = {
  /**
   * 创建监控定时任务
   */
  async generateSchedule(scheduleItem) {
    // 监控名称 时间片个数
    let { monitorName, timerNum } = scheduleItem;
    const cron = `0 0 0/${timerNum} * * *`;
    //  const cron = `*/20 * * * * ?`;
    // console.log(cron);
    // 生成任务唯一值
    const uuid = NodeUUID.v4();
    this.app.scheduleStacks[monitorName] = schedule.scheduleJob(
      uuid,
      cron,
      async () => {
        const locked = await this.app.redis.set(
          `${SCHEDULE_STACKS} + ${scheduleItem.monitorName}`,
          `${scheduleItem.monitorId}`,
          "EX",
          180,
          "NX"
        );
        if (!locked) return false;
        try {
          console.log("第一步", 333333333333333333333);
          await this.executeSchedule(scheduleItem);
        } catch (e) {
          await this.logger.info(
            "执行任务`%s`失败，时间：%s, 错误信息：%j",
            scheduleItem.monitorName,
            new Date().toLocaleString(),
            error
          );
        } finally {
          // 释放锁
          await this.app.redis.del(
            `${SCHEDULE_STACKS} + ${scheduleItem.monitorName}`
          );
        }
      }
    );
    // console.log(this.app.scheduleStacks);
  },
  /**
   * 创建dap数据同步定时任务
   */
  async generateDapSyncSchedule(scheduleItem) {
    let { DapCategory } = scheduleItem;
    const cron = `0 0 12 * * ?`; // 每天中午12点执行
    // 生成任务唯一值
    const uuid = NodeUUID.v4();
    this.app.dapscheduleStacks[DapCategory] = schedule.scheduleJob(
      uuid,
      cron,
      async () => {
        const locked = await this.app.redis.set(
          `${DAP_SCHEDULE_STACKS} + ${scheduleItem.DapCategory}`,
          `${scheduleItem.buryingId}`,
          "EX",
          180,
          "NX"
        );
        if (!locked) return false;
        try {
          console.log("第一步", 333333333333333333333);
          await this.executeDapSchedule(scheduleItem);
        } catch (e) {
          await this.logger.info(
            "执行任务`%s`失败，时间：%s, 错误信息：%j",
            scheduleItem.DapCategory,
            new Date().toLocaleString(),
            error
          );
        } finally {
          // 释放锁
          await this.app.redis.del(
            `${DAP_SCHEDULE_STACKS} + ${scheduleItem.DapCategory}`
          );
        }
      }
    );
    // console.log(this.app.scheduleStacks);
  },
  /**
   * 创建转化率监控定时任务
   */
  async generateTransferSchedule(transferItem) {
    // 监控名称 时间片个数
    let { transferId, modelId } = transferItem;
    const cron = `0 0 15 * * ?`; // 每天下午三点执行
    // const cron = `*/20 * * * * ?`;
    // console.log(cron);
    // 生成任务唯一值
    const uuid = NodeUUID.v4();
    this.app.transferStacks[`${modelId}&${transferId}`] = schedule.scheduleJob(
      uuid,
      cron,
      async () => {
        const locked = await this.app.redis.set(
          `${TRANSFE_STACKS} + ${transferItem.modelId} + ${transferItem.transferId}`,
          `${transferItem.transferId}`,
          "EX",
          180,
          "NX"
        );
        if (!locked) return false;
        try {
          console.log("第一步", 333333333333333333333);
          await this.logger.info(
          "调用执行转化率监控任务"
          );
          await this.executeTransferSchedule(transferItem);
        } catch (e) {
          await this.logger.info(
            "执行任务`%s`失败，时间：%s, 错误信息：%j",
            transferItem.modelId + transferItem.transferId,
            new Date().toLocaleString(),
            e
          );
        } finally {
          // 释放锁
          await this.app.redis.del(
            `${TRANSFE_STACKS} + ${transferItem.modelId} + ${transferItem.transferId}`
          );
        }
      }
    );
    // console.log(this.app.scheduleStacks);
  },

   /**
   * 创建证书监控定时任务
   */
  async generateCertificateMonitorSchedule(certificateMonitorItem) {
    // 监控名称 时间片个数
    let { certificateMonitorName, certificateMonitorId } = certificateMonitorItem;
    const cron = `0 0 */3 * *`; // 每隔三天执行
    // const cron = `*/20 * * * * ?`;
    // console.log(cron);
    // 生成任务唯一值
    const uuid = NodeUUID.v4();
    this.app.certificateStacks[`${certificateMonitorName}&${certificateMonitorId}`] = schedule.scheduleJob(
      uuid,
      cron,
      async () => {
        const locked = await this.app.redis.set(
          `${CERTIFICATE_STACKS} + ${certificateMonitorItem.certificateMonitorId} + ${certificateMonitorItem.certificateMonitorName}`,
          `${certificateMonitorItem.certificateMonitorId}`,
          "EX",
          180,
          "NX"
        );
        if (!locked) return false;
        try {
          await this.logger.info(
          "调用执行证书监控任务"
          );
          await this.executeCertificateMonitorSchedule(certificateMonitorItem);
        } catch (e) {
          await this.logger.info(
            "执行任务`%s`失败，时间：%s, 错误信息：%j",
            certificateMonitorItem.certificateMonitorId + certificateMonitorItem.certificateMonitorName,
            new Date().toLocaleString(),
            e
          );
        } finally {
          // 释放锁
          await this.app.redis.del(
            `${CERTIFICATE_STACKS} + ${certificateMonitorItem.certificateMonitorId} + ${certificateMonitorItem.certificateMonitorName}`
          );
        }
      }
    );
  },





  /**
   * 取消/停止Dap同步定时任务
   * @param {*} jobName 监控任务名
   */
  async cancelDapSchedule(jobName) {
    await this.app.redis.del(`${DAP_SCHEDULE_STACKS} + ${jobName}`);
    this.app.dapscheduleStacks[jobName] &&
      this.app.dapscheduleStacks[jobName].cancel();
  },
  /**
   * 取消/停止监控定时任务
   * @param {*} jobName 监控任务名
   */
  async cancelSchedule(jobName) {
    await this.app.redis.del(`${SCHEDULE_STACKS} + ${jobName}`);
    this.app.scheduleStacks[jobName] &&
      this.app.scheduleStacks[jobName].cancel();
  },
  /**
   * 取消/停止转化率监控定时任务
   * @param {*} modelId 模型id
   * @param {*} transferId 转化id
   */
  async cancelTransferSchedule(modelId, transferId) {
    await this.app.redis.del(`${TRANSFE_STACKS} + ${modelId} + ${transferId}`);
    this.app.transferStacks[`${modelId}&${transferId}`] &&
      this.app.transferStacks[`${modelId}&${transferId}`].cancel();
  },
   /**
   * 取消/停止证书监控定时任务
   * @param {*} certificateMonitorName  证书监控名称
   * @param {*} certificateMonitorId 证书监控id
   */
    async cancelCertificateMonitorSchedule(certificateMonitorName, certificateMonitorId) {
      await this.app.redis.del(`${CERTIFICATE_STACKS} + ${certificateMonitorId} + ${certificateMonitorName}`);
      this.app.certificateStacks[`${certificateMonitorName}&${certificateMonitorId}`] &&
        this.app.certificateStacks[`${certificateMonitorName}&${certificateMonitorId}`].cancel();
        // this.app.certificateStacks[`${certificateMonitorName}&${certificateMonitorId}`]
    },
  /**
   * 执行监控任务
   */
  async executeSchedule(scheduleItem) {
    console.log(scheduleItem);
    try {
      // 判断任务状态
      if (scheduleItem.onlineType === SCHEDULE_STATUS.STOP) {
        // 当任务处于下线状态时，取消当前执行
        // 任务容错，防止用户在调用下线接口时不是当前worker
        await this.cancelSchedule(scheduleItem.monitorName);
        await this.logger.info("执行任务`%s`时，任务状态为停止状态");
      } else {
        console.log("第三步", scheduleItem.monitorName, 333333333333333333333);
        scheduleItem.retryTimes = 0 // 初始化重试次数
        await this.service.scheduleService.handleCurlTest(scheduleItem);
      }
    } catch (error) {
      await this.logger.info(
        "执行任务`%s`失败，时间：%s, 错误信息：%j",
        scheduleItem.monitorName,
        new Date().toLocaleString(),
        error
      );
    }
  },
  /**
   * 执行dap数据同步任务
   */
  async executeDapSchedule(scheduleItem) {
    try {
      // 判断任务状态
      if (scheduleItem.DapSync === DAP_SYNC_STATUS.UNSYNC) {
        // 当DAP数据同步状态处于关闭时，取消当前执行
        // 任务容错，防止用户在调用下线接口时不是当前worker
        await this.cancelDapSchedule(scheduleItem.DapCategory);
        await this.logger.info("执行任务`%s`时，任务状态为关闭状态");
      } else {
        await this.service.pushImplaService.handlePushImpla(scheduleItem);
      }
    } catch (error) {
      await this.logger.info(
        "执行任务`%s`失败，时间：%s, 错误信息：%j",
        scheduleItem.DapCategory,
        new Date().toLocaleString(),
        error
      );
    }
  },
  /**
   * 执行转化率监控任务
   */
  async executeTransferSchedule(scheduleItem) {
    try {
      // 判断任务状态
      if (scheduleItem.isAlert === TRANSFER_STATUS.off) {
        // 当转化率监控报警状态处于关闭时，取消当前执行
        // 任务容错，防止用户调用不是当前worker
        await this.cancelTransferSchedule(
          scheduleItem.modelId,
          scheduleItem.transferId
        );
        await this.logger.info("执行任务`%s`时，任务状态为关闭状态");
      } else {
        await this.logger.info(`触发转化服务`);
        await this.service.transferService.transferMonitor(scheduleItem);
      }
    } catch (error) {
      await this.logger.info(
        "执行任务`%s`失败，时间：%s, 错误信息：%j",
        scheduleItem.modelId + scheduleItem.transferId,
        new Date().toLocaleString(),
        error
      );
    }
  },
  /**
   * 执行证书监控任务
   */
  async executeCertificateMonitorSchedule(scheduleItem) {
      try {
        // 判断任务状态
        if (scheduleItem.isOnline === CERTIFICATE_MONITOR_STATUS.off) {
          // 当证书监控报警状态处于关闭时，取消当前执行
          // 任务容错，防止用户调用不是当前worker
          await this.cancelCertificateMonitorSchedule(
            scheduleItem.certificateMonitorName,
            scheduleItem.certificateMonitorId
          );
          await this.logger.info("执行任务`%s`时，任务状态为关闭状态");
        } else {
          await this.service.certificateService.certificateMonitor(scheduleItem);
        }
      } catch (error) {
        await this.logger.info(
          "执行任务`%s`失败，时间：%s, 错误信息：%j",
          scheduleItem.certificateMonitorName + scheduleItem.certificateMonitorId,
          new Date().toLocaleString(),
          error
        );
      }
    },

  // 根据size拆分数组
  async splitArray(array, size) {
    let data = [];
    for (let i = 0; i < array.length; i += size) {
      data.push(array.slice(i, i + size));
    }
    return data;
  },
  // 除法
  async div(arg1, arg2) {
    var r1 = arg1.toString(),
      r2 = arg2.toString(),
      m,
      resultVal,
      d = arguments[2];
    m =
      (r2.split(".")[1] ? r2.split(".")[1].length : 0) -
      (r1.split(".")[1] ? r1.split(".")[1].length : 0);
    resultVal =
      (Number(r1.replace(".", "")) / Number(r2.replace(".", ""))) *
      Math.pow(10, m);
    return typeof d !== "number"
      ? Number(resultVal)
      : Number(resultVal.toFixed(parseInt(d)));
  },
  // 乘法
  async mul(arg1, arg2) {
    var r1 = arg1.toString(), // 将传入的数据转化为字符串
      r2 = arg2.toString(),
      m,
      resultVal,
      d = arguments[2];
    m =
      (r1.split(".")[1] ? r1.split(".")[1].length : 0) +
      (r2.split(".")[1] ? r2.split(".")[1].length : 0); // 获取两个数字的小数位数的和
    // 乘积的算法就是去掉小数点做整数相乘然后除去10的所有小数位的次方
    resultVal =
      (Number(r1.replace(".", "")) * Number(r2.replace(".", ""))) /
      Math.pow(10, m);

    return typeof d !== "number"
      ? Number(resultVal)
      : Number(resultVal.toFixed(parseInt(d)));
  },
  // 加法
  async add(arg1, arg2) {
    (arg1 = arg1.toString()), (arg2 = arg2.toString()); // 将传入的数据转化为字符串
    var arg1Arr = arg1.split("."), // 将小数的数据从小数点的位置拆开
      arg2Arr = arg2.split("."),
      d1 = arg1Arr.length == 2 ? arg1Arr[1] : "", // 获取第一个数的小数点的长度
      d2 = arg2Arr.length == 2 ? arg2Arr[1] : ""; // 获取第二个数的小数点的长度
    var maxLen = Math.max(d1.length, d2.length); // 获取小数点长度较大的值
    var m = Math.pow(10, maxLen); // 这里表示10的小数点长度次方 也就是说如果小数点长度为2 m的值就是100 如果小数点长度是3 m的值就是1000如果不懂请自行查找api
    var result = Number(((arg1 * m + arg2 * m) / m).toFixed(maxLen)); // 将小数转化为整数后相加在除掉两个数乘过的倍数然后去小数点较长的长度的小数位数
    var d = arguments[2]; // 第三个参数用户可以自行决定是否要传递 用来定义要保留的小数长度
    return typeof d === "number" ? Number(result.toFixed(d)) : result;
  },
  // 减法
  async sun(arg1, arg2) {
    return this.add(arg1, -arg2);
  },
};
