"use strict";

module.exports = (options) => {
  const notAuthPath = {
    "/public/login": true,
  };
  return async function auth(ctx, next) {
    if (notAuthPath[ctx.path]) {
      await next();
      return;
    }
    // 拿到传会数据的header 中的token值
    const token = ctx.request.header.authorization;
    const method = ctx.method.toLowerCase();
    if (!token) {
      // 可以添加白名单
      if (notAuthPath[ctx.path]) {
        await next();
      } else {
        ctx.body = {
          status: 401,
          msg: "登录过期，请重新登录",
        };
      }
    } else {
      // 当前token值存在
      let decode;

      // 解码token
      decode = await ctx.app.jwt.verify(
        token,
        ctx.app.config.jwt.secret,
        (err, decoded) => {
          if (err) {
            if (err.name === "TokenExpiredError") {
              // token过期
              return "TokenExpiredError";
            } else if (err.name === "JsonWebTokenError") {
              // 无效的token
              return "JsonWebTokenError";
            }
          } else {
            return decoded;
          }
        }
      );

      if (decode === "TokenExpiredError") {
        ctx.body = {
          status: 401,
          msg: "登录过期，请重新登录",
        };
        return;
      }

      if (decode === "JsonWebTokenError") {
        ctx.body = {
          status: 50012,
          msg: "token无效，请重新登录",
        };
        return;
      }
      await next();
    }
  };
};
