"use strict";

/**
 * @param {Egg.Application} app - egg application
 */
module.exports = (app) => {
  const { router, controller } = app;
  router.get("/", controller.home.index);
  router.get("/test", controller.home.test);
  router.get("/login", controller.home.login);
  // 公共模块
  router.post("/public/login", controller.public.login);
  // 用户模块
  router.get("/user/getUserList", controller.user.getUserList);
  router.post("/user/addUser", controller.user.addUser);
  router.post("/user/editPassword", controller.user.editPassword);
  router.post("/user/sendEmailCode", controller.user.sendEmailCode);
  router.post("/user/editEmail", controller.user.editEmail);
  router.get("/user/getUserInfo", controller.user.getUserInfo);
  router.post("/user/operateUser", controller.user.operateUser);

  // 权限管理模块
  router.get("/role/roleList", controller.role.roleList); // 获取角色列表
  router.post("/role/roleAdd", controller.role.roleAdd); // 新增角色
  router.post("/role/roleEdit", controller.role.roleEdit); // 编辑角色
  router.post("/role/roleDelete", controller.role.roleDelete); // 删除角色
  router.post("/role/roleMenuUpdate", controller.role.roleMenuConfig); // 角色权限配置
  router.get("/role/roleDetail", controller.role.roleDetail); // 获取角色详情

  // 项目模块
  router.get("/project/getProjectList", controller.project.getProjectList);
  router.post("/project/updateProject", controller.project.addProject);
  // 监控模块
  router.get("/monitor/list", controller.monitor.getMonitorList); // 获取监控任务列表
  router.post("/monitor/test", controller.monitor.testMonitor); // 测试新加的监控任务
  router.post("/monitor/update", controller.monitor.updateMonitor); // 更新监控任务
  router.post("/monitor/online/operate", controller.monitor.onlineMonitor); // 监控上线下线操作
  router.get("/monitor/history/list", controller.monitor.historyList); // 监控历史记录
  router.get("/monitor/line/chart", controller.monitor.failureMonitor); // 监控失败率

  // 埋点管理模块
  router.get("/burying/list", controller.burying.getBuryingList); // 获取埋点列表
  router.get("/burying/appNameList", controller.burying.getAppNameList); // 获取所有AppName列表
  router.post("/burying/delete", controller.burying.buryingDelete); // 删除埋点
  router.post("/burying/GAConfig/update", controller.burying.GAConfigUpdate); // ga埋点配置修改
  router.post("/burying/DapConfig/update", controller.burying.DapConfigUpdate); // Dap埋点配置修改
  router.post("/burying/baseInfo/update", controller.burying.baseInfoUpdate); // 埋点基本信息新增/修改
  router.get("/burying/detail", controller.burying.buryingDetail); // 获取埋点详情
  router.get("/burying/DapStatistics/list", controller.burying.DapStatistics); // 获取DN沙盘统计列表
  router.post("/burying/Dap/sync", controller.burying.DapSync); // 是否同步每日DAP数据
  router.get("/burying/impala/tasklist", controller.burying.impalaTasklist); // 查询impla任务队列
  router.post("/burying/impala/reloadSync", controller.burying.reloadSync); // 重新加入队列
  router.post("/burying/canOperateDap", controller.burying.canOperateDap); // 是否可对埋点操作
  router.get(
    "/burying/dapcategory/getBuryId",
    controller.burying.getBuryIdBydDapcategory
  ); // 根据dapcategory查询对应的id
  router.get("/burying/getBuryingStatics", controller.burying.buryingStatis); // 获取埋点列表的详细信息

  // 产品数据模型模块
  router.post("/productModel/add", controller.productModel.productModelAdd); // 产品模型新增
  router.post("/productModel/edit", controller.productModel.productModelEdit); // 产品模型编辑
  router.get("/productModel/view", controller.productModel.productModelView); // 产品模型详情
  router.post(
    "/productMdel/delete",
    controller.productModel.productModelDelete
  ); // 产品模型删除
  router.get("/productModel/list", controller.productModel.productModelList); // 产品模型列表
  router.get("/productModel/getTree", controller.productModel.getTree); // 获取树列表

  router.post(
    "/productModel/getNodeDetail",
    controller.productModel.getNodeDetail
  ); // 获取当前树节点的所有转化路径

  // 转化率监控模块
  router.get(
    "/transfer/getTransferListByModelId",
    controller.transfer.getTransferListByModelId
  ); // 根据modelId获取转化列表
  router.get("/transfer/list", controller.transfer.transferList); // 转化率监控列表
  router.post("/transfer/config", controller.transfer.transferConfig); // 转化率监控配置
  router.get("/transfer/detail", controller.transfer.transferDetail); // 转化率监控详情
  router.post("/transfer/online", controller.transfer.transferOnline); // 转化率监控报警上下线
  router.get("/transfer/isAlert", controller.transfer.transferIsAlert); // 当前modelId下是否有开启的转化率监控报警任务


  // 证书监控模块
  router.get("/certificateMonitor/list", controller.certificateMonitor.certificateMonitorList); // 证书监控列表
  router.post("/certificateMonitor/update", controller.certificateMonitor.certificateMonitorUpdate); // 新增 / 编辑证书监控
  router.post("/certificateMonitor/online", controller.certificateMonitor.certificateMonitorOnline); // 上线 / 下线证书监控
};
