"use strict";

const Service = require("egg").Service;
const dayjs = require("dayjs");
const https = require("https");
const crypto = require("crypto");
class CertificateService extends Service {
  async certificateMonitor(scheduleItem) {
    const { ctx } = this;
    const httpsAgent = new https.Agent({ keepAlive: true });
    const {
      certificateMonitorName,
      certificateMonitorDomain,
      certificateMonitorId,
    } = scheduleItem;
    const options = {
      host: certificateMonitorDomain,
      port: 443,
      method: "HEAD",
      agent: httpsAgent,
    };
    const req = https.request(options, async (res) => {
      const cert = res.socket.getPeerCertificate();
      const certValidTo = new Date(cert.valid_to);
      const now = new Date();
      const daysUntilExpiration = (certValidTo - now) / (1000 * 60 * 60 * 24);
      console.log(cert.valid_to);
      console.log(certValidTo, daysUntilExpiration);
      console.log(
        `Days until certificate for ${options.host} expires: ${daysUntilExpiration}`
      );
      httpsAgent.destroy();
      let certificateDays = null;
      if (Number(Math.ceil(daysUntilExpiration)) > 30) {
        // 到期时间大于30天 (不报警)
        certificateDays = `还有${Number(Math.ceil(daysUntilExpiration))}天到期`;
      } else if (Number(Math.ceil(daysUntilExpiration)) < 0) {
        // 过期时间大于0天 （报警）
        certificateDays = `已过期${Math.abs(
          Number(Math.ceil(daysUntilExpiration))
        )}天`;
      } else if (
        Number(Math.ceil(daysUntilExpiration)) >= 0 &&
        Number(Math.ceil(daysUntilExpiration)) <= 30
      ) {
        // 过期时间大于0天
        certificateDays = `还有${Number(Math.ceil(daysUntilExpiration))}天到期`;
      }
      const rows = {
        certificateDays,
      };
      const options1 = {
        where: { certificateMonitorId },
      };
      await this.app.mysql.update("certificate_monitor", rows, options1);
      if (Number(Math.ceil(daysUntilExpiration)) <= 30) { //  小于等于30天报警
        const subject = "证书监控预警";
        const markDown =
          `## ${subject} \n` +
          `> 监控域名：${certificateMonitorDomain} \n\n` +
          `> 监控名称: ${certificateMonitorName} \n\n` +
          `> 证书状态: ${certificateDays} \n\n`;
        const robot = await this.service.dingtalkService.DingTalk(
          "810f9c981cd511c1e94d5365393001951d2464ca46833c68aced2f93e53bd3c5",
          "SEC32eca948790b0a5c4efb6a87f2f3dc0d124affabf9e6620c79b28868b889c663"
        );
        const at = {
          // 用户@具体用户
          atMobiles: ["durant.shi@uptech.ai"],
          isAtAll: false,
        };
        robot.markdown(subject, markDown, at);
      }
    });
    req.on("error", (e) => {
      console.error(e);
    });
    req.end();
  }
}
module.exports = CertificateService;
