const Service = require("egg").Service;
const NodeUUID = require("node-uuid");
const dayjs = require("dayjs");
class pushImplaService extends Service {
  /**
   * 调用curl服务API（HTTP）调用impla微服务
   * @param { scheduleItem } 一个埋点cateory组
   */
  async handlePushImpla(scheduleItem) {
    const { ctx } = this;
    const implaPath = this.config.implaPath;
    const { buryingId, DapAppName, DapCategory } = scheduleItem;
    const burylist = await this.app.mysql.select("bury_list", {
      where: { buryingId: buryingId, type: 2 },
    });
    const events = burylist.map((item) => {
      return item.eventName;
    });
    // 如果该catoery下的事件组为空停止执行
    if (events && events.length === 0) {
      return;
    }
    // 如果events大于30 分批请求
    if (events && events.length > 30) {
      const splitArr = await ctx.helper.splitArray(events, 30);
      for (let i = 0; i < splitArr.length; i++) {
        try {
          const data = {
            batchId: NodeUUID.v4(),
            appName: DapAppName,
            category: DapCategory,
            opdate: dayjs(new Date()).subtract(1, "day").format("YYYY-MM-DD"),
            events: splitArr[i],
          };
          this.logger.info(
            `发起impla查询：时间：${new Date()} 事件：${splitArr[i]}`
          );
          await ctx.curl(`${implaPath}/pushQueryTask`, {
            method: "POST",
            contentType: "json",
            timeout: 10000,
            data: [{ ...data }],
            dataType: "json",
          });
        } catch (e) {
          this.logger.info(`发起impla查询失败：${e}`);
          console.log(e);
        }
      }
    } else {
      try {
        const data = {
          batchId: NodeUUID.v4(),
          appName: DapAppName,
          category: DapCategory,
          opdate: dayjs(new Date()).subtract(1, "day").format("YYYY-MM-DD"),
          events: events,
        };
        this.logger.info(`发起impla查询：时间：${new Date()} 事件：${events}`);
        await ctx.curl(`${implaPath}/pushQueryTask`, {
          method: "POST",
          contentType: "json",
          timeout: 10000,
          data: [{ ...data }],
          dataType: "json",
        });
      } catch (e) {
        this.logger.info(`发起impla查询失败：${e}`);
        console.log(e);
      }
    }
  }
}
module.exports = pushImplaService;
