"use strict";

const { Service } = require("egg");
const dayjs = require("dayjs");
const fs = require("fs");
const { MONITOR_LEVEL_STATUS } = require("../constants");

class ScheduleService extends Service {
  /**
   * 测试处理程序
   * @param {*} params 任务参数
   * @param {*} jobHandlerLog 日志
   */
  async testHandler(params, jobHandlerLog) {
    // 此处替换成具体业务代码
    await this.logger.info("我是测试任务，任务参数: %s", params);
  }
  // 具体逻辑
  // curl 链接 -> 结果更新到历史记录表中 -> 如果有错误的情况 发送邮件推送相关错误信息并钉钉推送
  async handleCurlTest(taskItem) {
    const { monitorUrl, contentType } = taskItem;
    try {
      const res = await this.service.curlService.tarsServerHttp(monitorUrl, {
        dataType: contentType,
      });
      // 重试三次报警
      if (res.status !== 200 || (res.data.code && res.data.code !== 0)) {
        taskItem.retryTimes++;
        if (taskItem.retryTimes < 4) {
          return this.handleCurlTest(taskItem);
        }
      }
      if (contentType === "json") {
        this.handleJSONResult(taskItem, res);
      } else if (contentType === "text") {
        this.handleTextResult(taskItem, res);
      }
    } catch (e) {
      console.log(e);
    }
  }
  // 检查是json格式的时候的匹配
  async handleJSONResult(taskItem, res) {
    let sendResult = null;
    const {
      monitorUrl,
      projectId,
      monitorName,
      monitorLevel,
      email,
      resultRules,
    } = taskItem;
    const subject = "前端页面监控报警";
    const text = "文本：这是一封测试邮件";
    const at = {
      // 用户@具体用户
      atMobiles: ["durant.shi@uptech.ai"],
      isAtAll: false,
    };
    // 邮件模板格式
    const html = `<h2>页面地址：${taskItem.monitorUrl}</h2><br/>
    <p>
      http状态： ${res.status}
    </p><br/>
    <p>
      监控级别：${taskItem.monitorLevel || "暂未分级"}
    </p><br/>
    <p>
      错误原因：${res.name || res.data.error || res.res.statusMessage}
    </p><br/>
    <p>
      重试次数：${taskItem.retryTimes}
    </p><br/>
    <p>
      报警时间：${dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss")}
    </p><br/>`;
    // 钉钉推送格式
    const title = "监控报警";
    const markDown =
      `## ${title} \n` +
      `> 页面地址：<font color="#FF5733">${taskItem.monitorUrl}</font> \n\n` +
      `> http状态：<font color="#FF5733">${res.status}</font> \n\n` +
      `> 监控级别：<font color="#FF5733">${taskItem.monitorLevel || "暂未分级"}</font> \n\n` +
      `> 错误原因：<font color="#FF5733">${res.name || res.data.error || res.res.statusMessage}</font> \n\n` +
      `> 重试次数：<font color="#FF5733">${taskItem.retryTimes}</font> \n\n` + 
      `> 报警时间：<font color="#FF5733">${dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss")}</font> \n\n`;
    if (res.status !== 200 || (res.data.code && res.data.code !== 0)) {
      try {
        const robot = await this.service.dingtalkService.DingTalk(
          (monitorLevel &&
            MONITOR_LEVEL_STATUS[monitorLevel]?.dingAccessToken) ||
            "4f6c3b4dd4cd48922405e0db18ca356a50598567c48d96f6acd04731973602c6",
          (monitorLevel && MONITOR_LEVEL_STATUS[monitorLevel]?.dingSecret) ||
            "SEC04644fba15d8bb07019a5570f03c0b67e4b4c337932e6372e76c38e53a5696e6"
        );
        await robot.markdown(title, markDown, at);
      } catch (e) {}
      const has_send = await this.service.tool.sendMail(
        email,
        subject,
        text,
        html
      );
      if (has_send) {
        sendResult = 1;
      } else {
        sendResult = 0;
      }
      try {
        await this.app.mysql.insert("task_history", {
          monitorName,
          monitorUrl,
          sendResult, // 邮件发送结果
          checkResult: 0, // 检查结果失败
          createTime: dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss"),
          projectId,
        });
      } catch (error) {}
    } else {
      if (resultRules) {
        if (res.data && JSON.stringify(res.data).match(resultRules)) {
          // 判断正确
          try {
            await this.app.mysql.insert("task_history", {
              monitorName,
              monitorUrl,
              sendResult: 2, // 邮件检查结果成功不发送邮件
              checkResult: 1, // 检查结果成功
              createTime: dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss"),
              projectId,
            });
          } catch (error) {
            console.log(error);
          }
        } else {
          try {
            const subject = "前端页面监控报警";
            const text = "文本：这是一封测试邮件";
            const at = {
              // 用户@具体用户
              atMobiles: ["durant.shi@uptech.ai"],
              isAtAll: false,
            };
            const title = "监控报警";
            const markDown =
              `## ${title} \n` +
              `> 页面地址：${taskItem.monitorUrl} \n\n` +
              `> 监控级别：${taskItem.monitorLevel || "暂未分级"} \n\n` +
              `> 监控详情：你所匹配的规则没有在该页面找到 \n\n` + 
              `> 重试次数：${taskItem.retryTimes} \n\n` +
              `> 报警时间：${dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss")} \n\n`;
            try {
              const robot = await this.service.dingtalkService.DingTalk(
                (monitorLevel &&
                  MONITOR_LEVEL_STATUS[monitorLevel]?.dingAccessToken) ||
                  "4f6c3b4dd4cd48922405e0db18ca356a50598567c48d96f6acd04731973602c6",
                (monitorLevel &&
                  MONITOR_LEVEL_STATUS[monitorLevel]?.dingSecret) ||
                  "SEC04644fba15d8bb07019a5570f03c0b67e4b4c337932e6372e76c38e53a5696e6"
              );
              await robot.markdown(title, markDown, at);
            } catch (e) {}
            const html = `<h2>页面地址：${taskItem.monitorUrl}</h2><br/>
              <p>
                监控级别：${taskItem.monitorLevel || "暂未分级"}
              </p><br/>
              <p>你所匹配的规则没有在此页面找到</p><br/>
              <p>
                重试次数：${taskItem.retryTimes}
              </p><br/>
              <p>
                报警时间：${dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss")}
              </p><br/>`;
            const has_send = await this.service.tool.sendMail(
              email,
              subject,
              text,
              html
            );
            if (has_send) {
              sendResult = 1;
            } else {
              sendResult = 0;
            }
          } catch (e) {
            sendResult = 0;
          }
          try {
            await this.app.mysql.insert("task_history", {
              monitorName,
              monitorUrl,
              sendResult, // 邮件发送结果
              checkResult: 0, // 检查结果失败
              createTime: dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss"),
              projectId,
            });
          } catch (error) {}
        }
      } else {
        try {
          await this.app.mysql.insert("task_history", {
            monitorName,
            monitorUrl,
            sendResult: 2, // 邮件检查结果成功不发送邮件
            checkResult: 1, // 检查结果成功
            createTime: dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss"),
            projectId,
          });
        } catch (error) {
          console.log(error);
        }
      }
    }
  }
  // 检查是text格式的时候的匹配
  async handleTextResult(taskItem, res) {
    let sendResult = null;
    const {
      monitorUrl,
      projectId,
      monitorName,
      monitorLevel,
      email,
      resultRules,
    } = taskItem;
    const subject = "前端页面监控报警";
    const text = "文本：这是一封测试邮件";
    const at = {
      // 用户@具体用户
      atMobiles: ["durant.shi@uptech.ai"],
      isAtAll: false,
    };
    // 邮件模板格式
    const html = `<h2>页面地址：${taskItem.monitorUrl}</h2><br/>
      <p>
       http状态： ${res.status}
      </p><br/>
      <p>
       监控级别：${taskItem.monitorLevel || "暂未分级"}
      </p><br/>
      <p>
       错误原因：${res.name || res.data.error || res.res.statusMessage}
      </p><br/>
      <p>
       重试次数：${taskItem.retryTimes }
      </p><br/>
      <p>
        报警时间：${dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss")}
      </p><br/>`;
    // 钉钉推送格式
    const title = "监控报警";
    const markDown =
      `## ${title} \n` +
      `> 页面地址：<font color="#FF5733">${taskItem.monitorUrl}</font> \n\n` +
      `> http状态：<font color="#FF5733">${res.status}</font> \n\n` +
      `> 监控级别：<font color="#FF5733">${taskItem.monitorLevel || "暂未分级"}</font> \n\n` +
      `> 错误原因：<font color="#FF5733">${res.name || res.data.error || res.res.statusMessage}</font> \n\n` +
      `> 重试次数：<font color="#FF5733">${taskItem.retryTimes}</font> \n\n` + 
      `> 报警时间：<font color="#FF5733">${dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss")}</font> \n\n`;
    if (res.status !== 200 || (res.data.code && res.data.code !== 0)) {
      try {
        const robot = await this.service.dingtalkService.DingTalk(
          (monitorLevel &&
            MONITOR_LEVEL_STATUS[monitorLevel]?.dingAccessToken) ||
            "4f6c3b4dd4cd48922405e0db18ca356a50598567c48d96f6acd04731973602c6",
          (monitorLevel && MONITOR_LEVEL_STATUS[monitorLevel]?.dingSecret) ||
            "SEC04644fba15d8bb07019a5570f03c0b67e4b4c337932e6372e76c38e53a5696e6"
        );
        await robot.markdown(title, markDown, at);
      } catch (e) {}
      const has_send = await this.service.tool.sendMail(
        email,
        subject,
        text,
        html
      );
      if (has_send) {
        sendResult = 1;
      } else {
        sendResult = 0;
      }
      try {
        await this.app.mysql.insert("task_history", {
          monitorName,
          monitorUrl,
          sendResult, // 邮件发送结果
          checkResult: 0, // 检查结果失败
          createTime: dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss"),
          projectId,
        });
      } catch (error) {}
    } else {
      if (resultRules) {
        if (res.data && res.data.match(resultRules)) {
          // 判断正确
          try {
            await this.app.mysql.insert("task_history", {
              monitorName,
              monitorUrl,
              sendResult: 2, // 邮件检查结果成功不发送邮件
              checkResult: 1, // 检查结果成功
              createTime: dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss"),
              projectId,
            });
          } catch (error) {
            console.log(error);
          }
        } else {
          try {
            const subject = "前端页面监控报警";
            const text = "文本：这是一封测试邮件";
            const at = {
              // 用户@具体用户
              atMobiles: ["durant.shi@uptech.ai"],
              isAtAll: false,
            };
            const title = "监控报警";
            const markDown =
              `## ${title} \n` +
              `> 页面地址：<font color="#FF5733">${taskItem.monitorUrl}</font> \n\n` +
              `> 监控级别：<font color="#FF5733">${taskItem.monitorLevel || "暂未分级"}</font> \n\n` +
              `> 监控详情：<font color="#FF5733">你所匹配的规则没有在该页面找到</font> \n\n`;
              `> 重试次数：<font color="#FF5733">${taskItem.retryTimes}</font> \n\n` + 
              `> 报警时间：<font color="#FF5733">${dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss")}</font> \n\n`;
            try {
              const robot = await this.service.dingtalkService.DingTalk(
                (monitorLevel &&
                  MONITOR_LEVEL_STATUS[monitorLevel]?.dingAccessToken) ||
                  "4f6c3b4dd4cd48922405e0db18ca356a50598567c48d96f6acd04731973602c6",
                (monitorLevel &&
                  MONITOR_LEVEL_STATUS[monitorLevel]?.dingSecret) ||
                  "SEC04644fba15d8bb07019a5570f03c0b67e4b4c337932e6372e76c38e53a5696e6"
              );
              await robot.markdown(title, markDown, at);
            } catch (e) {}
            const html = `<h2>页面地址：${taskItem.monitorUrl}</h2><br/>
              <p>
                监控级别：${taskItem.monitorLevel || "暂未分级"}
              </p><br/>
              <p>你所匹配的规则没有在此页面找到</p><br/>
              <p>
              重试次数：${taskItem.retryTimes }
             </p><br/>
             <p>
               报警时间：${dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss")}
             </p><br/>`;
            const has_send = await this.service.tool.sendMail(
              email,
              subject,
              text,
              html
            );
            if (has_send) {
              sendResult = 1;
            } else {
              sendResult = 0;
            }
          } catch (e) {
            sendResult = 0;
          }
          try {
            await this.app.mysql.insert("task_history", {
              monitorName,
              monitorUrl,
              sendResult, // 邮件发送结果
              checkResult: 0, // 检查结果失败
              createTime: dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss"),
              projectId,
            });
          } catch (error) {}
        }
      } else {
        try {
          await this.app.mysql.insert("task_history", {
            monitorName,
            monitorUrl,
            sendResult: 2, // 邮件检查结果成功不发送邮件
            checkResult: 1, // 检查结果成功
            createTime: dayjs(new Date()).format("YYYY-MM-DD HH:mm:ss"),
            projectId,
          });
        } catch (error) {
          console.log(error);
        }
      }
    }
  }
}

module.exports = ScheduleService;
